@@ -84,7 +84,7 @@ enum key_binding_type {
 typedef tll(char *) config_modifier_list_t;
 
 struct config_key_binding {
-    int action;  /* One of the varios bind_action_* enums from wayland.h */
+    int action;  /* One of the various bind_action_* enums from wayland.h */
     //struct config_key_modifiers modifiers;
     config_modifier_list_t modifiers;
     union {
@@ -262,7 +262,10 @@ struct config {
     struct {
         enum cursor_style style;
         enum cursor_unfocused_style unfocused_style;
-        bool blink;
+        struct {
+            bool enabled;
+            uint32_t rate_ms;
+        } blink;
         struct {
             uint32_t text;
             uint32_t cursor;
@@ -335,8 +338,12 @@ struct config {
         SELECTION_TARGET_BOTH
     } selection_target;
 
-    struct config_spawn_template notify;
-    bool notify_focus_inhibit;
+    struct {
+        struct config_spawn_template command;
+        struct config_spawn_template command_action_arg;
+        struct config_spawn_template close;
+        bool inhibit_when_focused;
+    } desktop_notifications;
 
     env_var_list_t env_vars;
 
