@@ -3,7 +3,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
 
 #if defined(_DEBUG)
  #include <stdio.h>
@@ -32,7 +31,12 @@
 static void
 sgr_reset(struct terminal *term)
 {
-    memset(&term->vt.attrs, 0, sizeof(term->vt.attrs));
+    term->vt.attrs = (struct attributes){0};
+    term->vt.underline = (struct underline_range_data){0};
+
+    term->bits_affecting_ascii_printer.underline_style = false;
+    term->bits_affecting_ascii_printer.underline_color = false;
+    term_update_ascii_printer(term);
 }
 
 static const char *
@@ -88,17 +92,58 @@ csi_sgr(struct terminal *term)
         case 1: term->vt.attrs.bold = true; break;
         case 2: term->vt.attrs.dim = true; break;
         case 3: term->vt.attrs.italic = true; break;
-        case 4: term->vt.attrs.underline = true; break;
+        case 4: {
+            term->vt.attrs.underline = true;
+            term->vt.underline.style = UNDERLINE_SINGLE;
+
+            if (unlikely(term->vt.params.v[i].sub.idx == 1)) {
+                enum underline_style style = term->vt.params.v[i].sub.value[0];
+
+                switch (style) {
+                default:
+                case UNDERLINE_NONE:
+                    term->vt.attrs.underline = false;
+                    term->vt.underline.style = UNDERLINE_NONE;
+                    term->bits_affecting_ascii_printer.underline_style = false;
+                    break;
+
+                case UNDERLINE_SINGLE:
+                case UNDERLINE_DOUBLE:
+                case UNDERLINE_CURLY:
+                case UNDERLINE_DOTTED:
+                case UNDERLINE_DASHED:
+                    term->vt.underline.style = style;
+                    term->bits_affecting_ascii_printer.underline_style =
+                        style > UNDERLINE_SINGLE;
+                    break;
+                }
+
+                term_update_ascii_printer(term);
+            }
+            break;
+        }
         case 5: term->vt.attrs.blink = true; break;
         case 6: LOG_WARN("ignored: rapid blink"); break;
         case 7: term->vt.attrs.reverse = true; break;
         case 8: term->vt.attrs.conceal = true; break;
         case 9: term->vt.attrs.strikethrough = true; break;
 
-        case 21: break; /* double-underline, not implemented */
+        case 21:
+            term->vt.attrs.underline = true;
+            term->vt.underline.style = UNDERLINE_DOUBLE;
+            term->bits_affecting_ascii_printer.underline_style = true;
+            term_update_ascii_printer(term);
+            break;
+
         case 22: term->vt.attrs.bold = term->vt.attrs.dim = false; break;
         case 23: term->vt.attrs.italic = false; break;
-        case 24: term->vt.attrs.underline = false; break;
+        case 24: {
+            term->vt.attrs.underline = false;
+            term->vt.underline.style = UNDERLINE_NONE;
+            term->bits_affecting_ascii_printer.underline_style = false;
+            term_update_ascii_printer(term);
+            break;
+        }
         case 25: term->vt.attrs.blink = false; break;
         case 26: break;  /* rapid blink, ignored */
         case 27: term->vt.attrs.reverse = false; break;
@@ -119,7 +164,8 @@ csi_sgr(struct terminal *term)
             break;
 
         case 38:
-        case 48: {
+        case 48:
+        case 58: {
             uint32_t color;
             enum color_source src;
 
@@ -194,7 +240,12 @@ csi_sgr(struct terminal *term)
                 break;
             }
 
-            if (param == 38) {
+            if (unlikely(param == 58)) {
+                term->vt.underline.color_src = src;
+                term->vt.underline.color = color;
+                term->bits_affecting_ascii_printer.underline_color = true;
+                term_update_ascii_printer(term);
+            } else if (param == 38) {
                 term->vt.attrs.fg_src = src;
                 term->vt.attrs.fg = color;
             } else {
@@ -226,6 +277,13 @@ csi_sgr(struct terminal *term)
             term->vt.attrs.bg_src = COLOR_DEFAULT;
             break;
 
+        case 59:
+            term->vt.underline.color_src = COLOR_DEFAULT;
+            term->vt.underline.color = 0;
+            term->bits_affecting_ascii_printer.underline_color = false;
+            term_update_ascii_printer(term);
+            break;
+
         /* Bright foreground colors */
         case 90:
         case 91:
@@ -478,6 +536,9 @@ decset_decrst(struct terminal *term, unsigned param, bool enable)
             tll_free(term->alt.scroll_damage);
             term_damage_view(term);
         }
+
+        term->bits_affecting_ascii_printer.sixels =
+            tll_length(term->grid->sixel_images) > 0;
         term_update_ascii_printer(term);
         break;
 
@@ -500,6 +561,13 @@ decset_decrst(struct terminal *term, unsigned param, bool enable)
         term->grapheme_shaping = enable;
         break;
 
+    case 2048:
+        if (enable)
+            term_enable_size_notifications(term);
+        else
+            term_disable_size_notifications(term);
+        break;
+
     case 8452:
         term->sixel.cursor_right_of_graphics = enable;
         break;
@@ -507,8 +575,10 @@ decset_decrst(struct terminal *term, unsigned param, bool enable)
     case 737769:
         if (enable)
             term_ime_enable(term);
-        else
+        else {
             term_ime_disable(term);
+            term->ime_reenable_after_url_mode = false;
+        }
         break;
 
     default:
@@ -585,6 +655,7 @@ decrqm(const struct terminal *term, unsigned param)
     case 2027: return term->conf->tweak.grapheme_width_method != GRAPHEME_WIDTH_DOUBLE
         ? DECRPM_PERMANENTLY_RESET
         : decrpm(term->grapheme_shaping);
+    case 2048: return decrpm(term->size_notifications);
     case 8452: return decrpm(term->sixel.cursor_right_of_graphics);
     case 737769: return decrpm(term_ime_is_enabled(term));
     }
@@ -629,6 +700,7 @@ xtsave(struct terminal *term, unsigned param)
     case 2004: term->xtsave.bracketed_paste = term->bracketed_paste; break;
     case 2026: term->xtsave.app_sync_updates = term->render.app_sync_updates.enabled; break;
     case 2027: term->xtsave.grapheme_shaping = term->grapheme_shaping; break;
+    case 2048: term->xtsave.size_notifications = term->size_notifications; break;
     case 8452: term->xtsave.sixel_cursor_right_of_graphics = term->sixel.cursor_right_of_graphics; break;
     case 737769: term->xtsave.ime = term_ime_is_enabled(term); break;
     }
@@ -672,6 +744,7 @@ xtrestore(struct terminal *term, unsigned param)
     case 2004: enable = term->xtsave.bracketed_paste; break;
     case 2026: enable = term->xtsave.app_sync_updates; break;
     case 2027: enable = term->xtsave.grapheme_shaping; break;
+    case 2048: enable = term->xtsave.size_notifications; break;
     case 8452: enable = term->xtsave.sixel_cursor_right_of_graphics; break;
     case 737769: enable = term->xtsave.ime; break;
 
@@ -1114,6 +1187,7 @@ csi_dispatch(struct terminal *term, uint8_t final)
             if (param == 4) {
                 /* Insertion Replacement Mode (IRM) */
                 term->insert_mode = sm;
+                term->bits_affecting_ascii_printer.insert_mode = sm;
                 term_update_ascii_printer(term);
                 break;
             }
@@ -1665,7 +1739,7 @@ csi_dispatch(struct terminal *term, uint8_t final)
             switch (param) {
             case 0: /* blinking block, but we use it to reset to configured default */
                 term->cursor_style = term->conf->cursor.style;
-                term->cursor_blink.deccsusr = term->conf->cursor.blink;
+                term->cursor_blink.deccsusr = term->conf->cursor.blink.enabled;
                 term_cursor_blink_update(term);
                 break;
 
@@ -1983,6 +2057,85 @@ csi_dispatch(struct terminal *term, uint8_t final)
         break; /* private[0] == ‘$’ */
     }
 
+    case '#': {
+        switch (final) {
+        case 'P': { /* XTPUSHCOLORS */
+            int slot = vt_param_get(term, 0, 0);
+
+            /* Pm == 0, "push" (what xterm does is take take the
+               *current* slot + 1, even if that's in the middle of the
+               stack, and overwrites whatever is already in that
+               slot) */
+            if (slot == 0)
+                slot = term->color_stack.idx + 1;
+
+            if (term->color_stack.size < slot) {
+                const size_t new_size = slot;
+                term->color_stack.stack = xrealloc(
+                    term->color_stack.stack,
+                    new_size * sizeof(term->color_stack.stack[0]));
+
+                /* Initialize new slots (except the selected slot,
+                   which is done below) */
+                xassert(new_size > 0);
+                for (size_t i = term->color_stack.size; i < new_size - 1; i++) {
+                    memcpy(&term->color_stack.stack[i], &term->colors,
+                           sizeof(term->colors));
+                }
+                term->color_stack.size = new_size;
+            }
+
+            xassert(slot > 0);
+            xassert(slot <= term->color_stack.size);
+            term->color_stack.idx = slot;
+            memcpy(&term->color_stack.stack[slot - 1], &term->colors,
+                   sizeof(term->colors));
+            break;
+        }
+
+        case 'Q': {  /* XTPOPCOLORS */
+            int slot = vt_param_get(term, 0, 0);
+
+            /* Pm == 0, "pop" (what xterm does is copy colors from the
+              *current* slot, *and* decrease the current slot index,
+              even if that's in the middle of the stack) */
+            if (slot == 0)
+                slot = term->color_stack.idx;
+
+            if (slot > 0 && slot <= term->color_stack.size) {
+                memcpy(&term->colors, &term->color_stack.stack[slot - 1],
+                       sizeof(term->colors));
+                term->color_stack.idx = slot - 1;
+
+                /* Assume a full palette switch *will* affect almost
+                   all cells. The alternative is to call
+                   term_damage_color() for all 256 palette entries
+                   *and* the default fg/bg (256 + 2 calls in total) */
+                term_damage_view(term);
+                term_damage_margins(term);
+            } else if (slot == 0) {
+                LOG_ERR("XTPOPCOLORS: cannot pop beyond the first element");
+            } else {
+                LOG_ERR(
+                    "XTPOPCOLORS: invalid color slot: %d "
+                    "(stack has %zu slots, current slot is %zu)",
+                    vt_param_get(term, 0, 0),
+                    term->color_stack.size, term->color_stack.idx);
+            }
+            break;
+        }
+
+        case 'R': {  /* XTREPORTCOLORS */
+            char reply[64];
+            int n = xsnprintf(reply, sizeof(reply), "\033[?%zu;%zu#Q",
+                              term->color_stack.idx, term->color_stack.size);
+            term_to_slave(term, reply, n);
+            break;
+        }
+        }
+        break; /* private[0] == '#' */
+    }
+
     case 0x243f:  /* ?$ */
         switch (final) {
         case 'p': {
