@@ -255,7 +255,7 @@ instantiate_offset(struct buffer_private *buf, off_t new_offset)
 
     void *mmapped = MAP_FAILED;
     struct wl_buffer *wl_buf = NULL;
-    pixman_image_t **pix = xcalloc(buf->public.pix_instances, sizeof(*pix));
+    pixman_image_t **pix = xcalloc(buf->public.pix_instances, sizeof(pix[0]));
 
     mmapped = (uint8_t *)pool->real_mmapped + new_offset;
 
@@ -350,7 +350,7 @@ get_new_buffers(struct buffer_chain *chain, size_t count,
         "foot-wayland-shm-buffer-pool",
         MFD_CLOEXEC | MFD_ALLOW_SEALING | MFD_NOEXEC_SEAL);
 
-    if (pool_fd < 0 && errno == EINVAL) {
+    if (pool_fd < 0 && errno == EINVAL && MFD_NOEXEC_SEAL != 0) {
         pool_fd = memfd_create(
             "foot-wayland-shm-buffer-pool", MFD_CLOEXEC | MFD_ALLOW_SEALING);
     }
@@ -564,7 +564,9 @@ shm_get_buffer(struct buffer_chain *chain, int width, int height, bool with_alph
     tll_foreach(chain->bufs, it) {
         struct buffer_private *buf = it->item;
 
-        if (buf->public.width != width || buf->public.height != height) {
+        if (buf->public.width != width || buf->public.height != height ||
+            with_alpha != buf->with_alpha)
+        {
             LOG_DBG("purging mismatching buffer %p", (void *)buf);
             if (buffer_unref_no_remove_from_chain(buf))
                 tll_remove(chain->bufs, it);
