@@ -54,7 +54,13 @@ sixel_put
 sixel_init(struct terminal *term, int p1, int p2, int p3)
 {
     /*
-     * P1: pixel aspect ratio - unimplemented
+     * P1: pixel aspect ratio
+     *  - 0,1   - 2:1
+     *  - 2     - 5:1
+     *  - 3,4   - 3:1
+     *  - 5,6   - 2:1
+     *  - 7,8,9 - 1:1
+     *
      * P2: background color mode
      *  - 0|2: empty pixels use current background color
      *  - 1:   empty pixels remain at their current color (i.e. transparent)
@@ -414,6 +420,8 @@ sixel_scroll_up(struct terminal *term, int rows)
         }
     }
 
+    term->bits_affecting_ascii_printer.sixels =
+        tll_length(term->grid->sixel_images) > 0;
     term_update_ascii_printer(term);
     verify_sixels(term);
 }
@@ -438,6 +446,8 @@ sixel_scroll_down(struct terminal *term, int rows)
             break;
     }
 
+    term->bits_affecting_ascii_printer.sixels =
+        tll_length(term->grid->sixel_images) > 0;
     term_update_ascii_printer(term);
     verify_sixels(term);
 }
@@ -846,6 +856,8 @@ sixel_overwrite_by_rectangle(
     } else
         _sixel_overwrite_by_rectangle(term, start, col, height, width, NULL, NULL);
 
+    term->bits_affecting_ascii_printer.sixels =
+        tll_length(term->grid->sixel_images) > 0;
     term_update_ascii_printer(term);
 }
 
@@ -902,6 +914,8 @@ sixel_overwrite_by_row(struct terminal *term, int _row, int col, int width)
         }
     }
 
+    term->bits_affecting_ascii_printer.sixels =
+        tll_length(term->grid->sixel_images) > 0;
     term_update_ascii_printer(term);
 }
 
@@ -1381,6 +1395,9 @@ sixel_unhook(struct terminal *term)
     LOG_DBG("you now have %zu sixels in current grid",
             tll_length(term->grid->sixel_images));
 
+
+    term->bits_affecting_ascii_printer.sixels =
+        tll_length(term->grid->sixel_images) > 0;
     term_update_ascii_printer(term);
     render_refresh(term);
 }
