@@ -20,6 +20,7 @@
 #include "fdm.h"
 #include "key-binding.h"
 #include "macros.h"
+#include "notify.h"
 #include "reaper.h"
 #include "shm.h"
 #include "wayland.h"
@@ -99,19 +100,58 @@ struct damage {
     uint16_t lines;
 };
 
-struct row_uri_range {
-    int start;
-    int end;
+struct uri_range_data {
     uint64_t id;
     char *uri;
 };
 
+enum underline_style {
+    UNDERLINE_NONE,
+    UNDERLINE_SINGLE,  /* Legacy underline */
+    UNDERLINE_DOUBLE,
+    UNDERLINE_CURLY,
+    UNDERLINE_DOTTED,
+    UNDERLINE_DASHED,
+};
+
+struct underline_range_data {
+    enum underline_style style;
+    enum color_source color_src;
+    uint32_t color;
+};
+
+union row_range_data {
+    struct uri_range_data uri;
+    struct underline_range_data underline;
+};
+
+struct row_range {
+    int start;
+    int end;
+
+    union {
+        /* This is just an expanded union row_range_data, but
+         * anonymous, so that we don't have to write range->u.uri.id,
+         * but can instead do range->uri.id */
+        union {
+            struct uri_range_data uri;
+            struct underline_range_data underline;
+        };
+        union row_range_data data;
+    };
+};
+
+struct row_ranges {
+    struct row_range *v;
+    int size;
+    int count;
+};
+
+enum row_range_type {ROW_RANGE_URI, ROW_RANGE_UNDERLINE};
+
 struct row_data {
-    struct {
-        struct row_uri_range *v;
-        uint32_t size;
-        uint32_t count;
-    } uri_ranges;
+    struct row_ranges uri_ranges;
+    struct row_ranges underline_ranges;
 };
 
 struct row {
@@ -260,6 +300,8 @@ struct vt {
         char *uri;
     } osc8;
 
+    struct underline_range_data underline;
+
     struct {
         uint8_t *data;
         size_t size;
@@ -352,12 +394,36 @@ struct url {
 };
 typedef tll(struct url) url_list_t;
 
+
+struct colors {
+    uint32_t fg;
+    uint32_t bg;
+    uint32_t table[256];
+    uint16_t alpha;
+    uint32_t cursor_fg;  /* Text color */
+    uint32_t cursor_bg;  /* cursor color */
+    uint32_t selection_fg;
+    uint32_t selection_bg;
+    bool use_custom_selection;
+};
+
 struct terminal {
     struct fdm *fdm;
     struct reaper *reaper;
     const struct config *conf;
 
     void (*ascii_printer)(struct terminal *term, char32_t c);
+    union {
+        struct {
+            bool sixels:1;
+            bool osc8:1;
+            bool underline_style:1;
+            bool underline_color:1;
+            bool insert_mode:1;
+            bool charset:1;
+        };
+        uint8_t value;
+    } bits_affecting_ascii_printer;
 
     pid_t slave;
     int ptmx;
@@ -476,6 +542,8 @@ struct terminal {
         bool app_sync_updates:1;
         bool grapheme_shaping:1;
 
+        bool size_notifications:1;
+
         bool sixel_display_mode:1;
         bool sixel_private_palette:1;
         bool sixel_cursor_right_of_graphics:1;
@@ -511,15 +579,13 @@ struct terminal {
     int cell_width;  /* pixels per cell, x-wise */
     int cell_height; /* pixels per cell, y-wise */
 
+    struct colors colors;
+
     struct {
-        uint32_t fg;
-        uint32_t bg;
-        uint32_t table[256];
-        uint16_t alpha;
-        uint32_t selection_fg;
-        uint32_t selection_bg;
-        bool use_custom_selection;
-    } colors;
+        struct colors *stack;
+        size_t idx;
+        size_t size;
+    } color_stack;
 
     enum cursor_style cursor_style;
     struct {
@@ -528,10 +594,6 @@ struct terminal {
         int fd;
         enum { CURSOR_BLINK_ON, CURSOR_BLINK_OFF } state;
     } cursor_blink;
-    struct {
-        uint32_t text;
-        uint32_t cursor;
-    } cursor_color;
 
     struct {
         enum selection_kind kind;
@@ -714,11 +776,18 @@ struct terminal {
     char32_t url_keys[5];
     bool urls_show_uri_on_jump_label;
     struct grid *url_grid_snapshot;
+    bool ime_reenable_after_url_mode;
 
 #if defined(FOOT_IME_ENABLED) && FOOT_IME_ENABLED
     bool ime_enabled;
 #endif
 
+    struct {
+        bool active;
+        int count;
+        char32_t character;
+    } unicode_mode;
+
     struct {
         bool in_progress;
         bool client_has_terminated;
@@ -730,10 +799,19 @@ struct terminal {
         void *cb_data;
     } shutdown;
 
+    /* State, to handle chunked notifications */
+    struct notification kitty_notification;
+
+    /* Currently active notifications, from foot's perspective (their
+       notification helper processes are still running) */
+    tll(struct notification) active_notifications;
+    struct notification_icon notification_icons[32];
+
     char *foot_exe;
     char *cwd;
 
     bool grapheme_shaping;
+    bool size_notifications;
 };
 
 struct config;
@@ -779,6 +857,7 @@ void term_damage_view(struct terminal *term);
 
 void term_damage_cursor(struct terminal *term);
 void term_damage_margins(struct terminal *term);
+void term_damage_color(struct terminal *term, enum color_source src, int idx);
 
 void term_reset_view(struct terminal *term);
 
@@ -879,6 +958,10 @@ void term_osc8_close(struct terminal *term);
 bool term_ptmx_pause(struct terminal *term);
 bool term_ptmx_resume(struct terminal *term);
 
+void term_enable_size_notifications(struct terminal *term);
+void term_disable_size_notifications(struct terminal *term);
+void term_send_size_notification(struct terminal *term);
+
 static inline void term_reset_grapheme_state(struct terminal *term)
 {
 #if defined(FOOT_GRAPHEME_CLUSTERING)
