@@ -1,11 +1,12 @@
 #include "wayland.h"
 
+#include <errno.h>
+#include <fcntl.h>
+#include <locale.h>
+#include <poll.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
-#include <poll.h>
-#include <fcntl.h>
 
 #include <sys/timerfd.h>
 #include <sys/epoll.h>
@@ -13,6 +14,8 @@
 #include <cursor-shape-v1.h>
 #include <wayland-client.h>
 #include <wayland-cursor.h>
+#include <xkbcommon/xkbcommon.h>
+#include <xkbcommon/xkbcommon-keysyms.h>
 #include <xkbcommon/xkbcommon-compose.h>
 
 #include <tllist.h>
@@ -1117,9 +1120,20 @@ handle_global(void *data, struct wl_registry *registry,
         if (!verify_iface_version(interface, version, required))
             return;
 
+#if defined(WL_SHM_RELEASE_SINCE_VERSION)
+        const uint32_t preferred = WL_SHM_RELEASE_SINCE_VERSION;
+#else
+        const uint32_t preferred = required;
+#endif
+
         wayl->shm = wl_registry_bind(
-            wayl->registry, name, &wl_shm_interface, required);
+            wayl->registry, name, &wl_shm_interface, min(version, preferred));
         wl_shm_add_listener(wayl->shm, &shm_listener, wayl);
+#if defined(WL_SHM_RELEASE_SINCE_VERSION)
+        wayl->use_shm_release = version >= WL_SHM_RELEASE_SINCE_VERSION;
+#else
+        wayl->use_shm_release = false;
+#endif
     }
 
     else if (streq(interface, xdg_wm_base_interface.name)) {
@@ -1161,6 +1175,12 @@ handle_global(void *data, struct wl_registry *registry,
         if (!verify_iface_version(interface, version, required))
             return;
 
+#if defined(WL_POINTER_AXIS_VALUE120_SINCE_VERSION)
+        const uint32_t preferred = WL_POINTER_AXIS_VALUE120_SINCE_VERSION;
+#else
+        const uint32_t preferred = required;
+#endif
+
         int repeat_fd = timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
         if (repeat_fd == -1) {
             LOG_ERRNO("failed to create keyboard repeat timer FD");
@@ -1168,7 +1188,7 @@ handle_global(void *data, struct wl_registry *registry,
         }
 
         struct wl_seat *wl_seat = wl_registry_bind(
-            wayl->registry, name, &wl_seat_interface, required);
+            wayl->registry, name, &wl_seat_interface, min(version, preferred));
 
         tll_push_back(wayl->seats, ((struct seat){
                     .wayl = wayl,
@@ -1189,6 +1209,19 @@ handle_global(void *data, struct wl_registry *registry,
             return;
         }
 
+        seat->kbd.xkb = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
+        if (seat->kbd.xkb != NULL) {
+            seat->kbd.xkb_compose_table = xkb_compose_table_new_from_locale(
+                seat->kbd.xkb, setlocale(LC_CTYPE, NULL), XKB_COMPOSE_COMPILE_NO_FLAGS);
+
+            if (seat->kbd.xkb_compose_table != NULL) {
+                seat->kbd.xkb_compose_state = xkb_compose_state_new(
+                    seat->kbd.xkb_compose_table, XKB_COMPOSE_STATE_NO_FLAGS);
+            } else {
+                LOG_WARN("failed to instantiate compose table; dead keys (compose) will not work");
+            }
+        }
+
         seat_add_data_device(seat);
         seat_add_primary_selection(seat);
         seat_add_text_input(seat);
@@ -1320,6 +1353,16 @@ handle_global(void *data, struct wl_registry *registry,
             wayl->registry, name, &wp_cursor_shape_manager_v1_interface, required);
     }
 
+    else if (streq(interface, wp_single_pixel_buffer_manager_v1_interface.name)) {
+        const uint32_t required = 1;
+        if (!verify_iface_version(interface, version, required))
+            return;
+
+        wayl->single_pixel_manager = wl_registry_bind(
+            wayl->registry, name,
+            &wp_single_pixel_buffer_manager_v1_interface, required);
+    }
+
 #if defined(FOOT_IME_ENABLED) && FOOT_IME_ENABLED
     else if (streq(interface, zwp_text_input_manager_v3_interface.name)) {
         const uint32_t required = 1;
@@ -1636,6 +1679,8 @@ wayl_destroy(struct wayland *wayl)
         zwp_text_input_manager_v3_destroy(wayl->text_input_manager);
 #endif
 
+    if (wayl->single_pixel_manager != NULL)
+        wp_single_pixel_buffer_manager_v1_destroy(wayl->single_pixel_manager);
     if (wayl->fractional_scale_manager != NULL)
         wp_fractional_scale_manager_v1_destroy(wayl->fractional_scale_manager);
     if (wayl->viewporter != NULL)
@@ -1656,8 +1701,14 @@ wayl_destroy(struct wayland *wayl)
         wl_data_device_manager_destroy(wayl->data_device_manager);
     if (wayl->primary_selection_device_manager != NULL)
         zwp_primary_selection_device_manager_v1_destroy(wayl->primary_selection_device_manager);
-    if (wayl->shm != NULL)
-        wl_shm_destroy(wayl->shm);
+    if (wayl->shm != NULL) {
+#if defined(WL_SHM_RELEASE_SINCE_VERSION)
+        if (wayl->use_shm_release)
+            wl_shm_release(wayl->shm);
+        else
+#endif
+            wl_shm_destroy(wayl->shm);
+    }
     if (wayl->sub_compositor != NULL)
         wl_subcompositor_destroy(wayl->sub_compositor);
     if (wayl->compositor != NULL)
@@ -1773,8 +1824,7 @@ wayl_win_init(struct terminal *term, const char *token)
     wl_surface_commit(win->surface.surf);
 
     /* Complete XDG startup notification */
-    if (token && wayl->xdg_activation != NULL)
-        xdg_activation_v1_activate(wayl->xdg_activation, token, win->surface.surf);
+    wayl_activate(wayl, win, token);
 
     if (!wayl_win_subsurface_new(win, &win->overlay, false)) {
         LOG_ERR("failed to create overlay surface");
@@ -2058,6 +2108,7 @@ surface_scale_explicit_width_height(
             }
         }
 
+        xassert(surf->viewport != NULL);
         wl_surface_set_buffer_scale(surf->surf, 1);
         wp_viewport_set_destination(
             surf->viewport, roundf(width / scale), roundf(height / scale));
@@ -2204,7 +2255,7 @@ wayl_win_subsurface_new_with_custom_parent(
     }
 
     struct wp_viewport *viewport = NULL;
-    if (wayl->fractional_scale_manager != NULL &&  wayl->viewporter != NULL) {
+    if (wayl->viewporter != NULL) {
         viewport = wp_viewporter_get_viewport(wayl->viewporter, main_surface);
         if (viewport == NULL) {
             LOG_ERR("failed to instantiate viewport for sub-surface");
@@ -2325,3 +2376,15 @@ wayl_get_activation_token(
     xdg_activation_token_v1_commit(token);
     return true;
 }
+
+void
+wayl_activate(struct wayland *wayl, struct wl_window *win, const char *token)
+{
+    if (wayl->xdg_activation == NULL)
+        return;
+
+    if (token == NULL)
+        return;
+
+    xdg_activation_v1_activate(wayl->xdg_activation, token, win->surface.surf);
+}
